package de.ellpeck.actuallyadditions.common.blocks.render;

import com.mojang.blaze3d.matrix.MatrixStack;
import de.ellpeck.actuallyadditions.api.ActuallyAdditionsAPI;
import de.ellpeck.actuallyadditions.api.recipe.EmpowererRecipe;
import de.ellpeck.actuallyadditions.common.ActuallyAdditions;
import de.ellpeck.actuallyadditions.common.tile.TileEntityEmpowerer;
import de.ellpeck.actuallyadditions.common.util.AssetUtil;
import de.ellpeck.actuallyadditions.common.util.StackUtil;
import net.minecraft.client.renderer.IRenderTypeBuffer;
import net.minecraft.client.renderer.Quaternion;
import net.minecraft.client.renderer.tileentity.TileEntityRenderer;
import net.minecraft.client.renderer.tileentity.TileEntityRendererDispatcher;
import net.minecraft.item.BlockItem;
import net.minecraft.item.ItemStack;
import net.minecraft.util.Direction;
import net.minecraft.util.Util;
import net.minecraft.util.math.BlockPos;

// todo: migrate to client package
public class RenderEmpowerer extends TileEntityRenderer<TileEntityEmpowerer> {
    public RenderEmpowerer(TileEntityRendererDispatcher rendererDispatcherIn) {
        super(rendererDispatcherIn);
    }

    @Override
    public void render(TileEntityEmpowerer tile, float partialTicks, MatrixStack matrices, IRenderTypeBuffer bufferIn, int combinedLightIn, int combinedOverlayIn) {
//        BlockPos pos = tile.getPos();
        ItemStack stack = tile.inv.getStackInSlot(0);
        if (StackUtil.isValid(stack)) {
            matrices.push();
//            matrices.translate((float) pos.getX() + 0.5F, (float) pos.getY() + 1F, (float) pos.getZ() + 0.5F);

            double boop = Util.nanoTime() / 800D;
            matrices.translate(0D, Math.sin(boop % (2 * Math.PI)) * 0.065, 0D);
            matrices.rotate(new Quaternion((float) (boop * 40D % 360), 0, 1, 0));

            float scale = stack.getItem() instanceof BlockItem ? 0.85F : 0.65F;
            matrices.scale(scale, scale, scale);
            try {
                AssetUtil.renderItemInWorld(stack);
            } catch (Exception e) {
                ActuallyAdditions.LOGGER.error("Something went wrong trying to render an item in an empowerer! The item is {}!", stack.getItem().getRegistryName(), e);
            }

            matrices.pop();
        }

        int index = tile.recipeForRenderIndex;
        if (index >= 0 && ActuallyAdditionsAPI.EMPOWERER_RECIPES.size() > index) {
            EmpowererRecipe recipe = ActuallyAdditionsAPI.EMPOWERER_RECIPES.get(index);
            if (recipe != null) {
                Direction.Plane.HORIZONTAL.iterator().forEachRemaining(e -> {
                    BlockPos offset = tile.getPos().offset(e, 3);

                    AssetUtil.renderLaser(tile.getPos().getX() + 0.5, tile.getPos().getY() + 0.5, tile.getPos().getZ() + 0.5, offset.getX() + 0.5, offset.getY() + 0.95, offset.getZ() + 0.5, 80, 1F, 0.1F, recipe.getParticleColors());
                });

// original code of the above if it doesn't work
// @since 1.15-port
//                for (int i = 0; i < Direction.Plane.HORIZONTAL; i++) {
//                    EnumFacing facing = EnumFacing.HORIZONTALS[i];
//                    BlockPos offset = tile.getPos().offset(facing, 3);
//
//                    AssetUtil.renderLaser(tile.getPos().getX() + 0.5, tile.getPos().getY() + 0.5, tile.getPos().getZ() + 0.5, offset.getX() + 0.5, offset.getY() + 0.95, offset.getZ() + 0.5, 80, 1F, 0.1F, recipe.getParticleColors());
//                }
            }
        }
    }
}
